@@ -12,7 +12,7 @@ use Scalar::Util qw(blessed weaken);
 
 use constant DEBUG => $ENV{MOJO_RUN3_DEBUG} && 1;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 our @SAFE_SIG = grep {
   !m!^(NUM\d+|__[A-Z0-9]+__|ALL|CATCHALL|DEFER|HOLD|IGNORE|MAX|PAUSE|RTMAX|RTMIN|SEGV|SETS)$!
@@ -202,6 +202,7 @@ sub _start_parent {
 
   Mojo::IOLoop::ReadWriteFork::SIGCHLD->singleton->waitpid(
     $self->{pid} => sub {
+      return unless $self;
       $self->{status} = $_[0];
       $self->_maybe_finish('child');
     }
@@ -233,6 +234,8 @@ sub _write {
   }
 }
 
+sub DESTROY { shift->_cleanup unless ${^GLOBAL_PHASE} eq 'DESTRUCT' }
+
 1;
 
 =encoding utf8
